#include <maya/MQtUtil.h>
#include <QtCore/QPointer>
#include <QtGui/QMouseEvent>
#include <QtGui/QApplication>
#include <QtOpenGL/QGLWidget>
#include "GLWidget.h"

#include <maya/MIOStream.h>
#include <math.h>
#include <stdlib.h>

#include <maya/MFnPlugin.h>
#include <maya/MString.h>
#include <maya/MGlobal.h>
#include <maya/M3dView.h>
#include <maya/MDagPath.h>
#include <maya/MCursor.h>
 
#include <maya/MItSelectionList.h>
#include <maya/MSelectionList.h>

#include <maya/MPxContextCommand.h>
#include <maya/MPxContext.h>
#include <maya/MEvent.h>

#include <maya/MPoint.h>

#if defined(OSMac_MachO_)
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#endif

const char titleString[] = "Marquee Tool";
const char helpString[] = "Click with left button or drag with middle button to select";

//////////////////////////////////////////////
// The user Context
//////////////////////////////////////////////
class marqueeContext : public MPxContext
{
public:
					marqueeContext();

	virtual void	toolOnSetup( MEvent & event );
	virtual void    toolOffCleanup( );

	virtual MStatus	doPress( MEvent & event );
	virtual MStatus	doDrag( MEvent & event );
	virtual MStatus	doRelease( MEvent & event );
	virtual MStatus doEnterRegion(MEvent &event);

	virtual void	getClassName( MString& name ) const;

	static void		cleanup();
	static QPointer<CoolWidget>	coolwdg;

private:
	short					pressX, pressY;
	short					releaseX, releaseY;
	int						clickBoxSize;

	MGlobal::ListAdjustment	listAdjustment;
	M3dView					view;
};

//	Destroy the button window, if it still exists.
void marqueeContext::cleanup()
{
	if (!coolwdg.isNull()) delete coolwdg;
}

//	We store a pointer to the button window in a static QPointer so that we
//	can destroy it if the plugin is unloaded. The QPointer will
//	automatically set itself to zero if the button window is destroyed
//	for any reason.
QPointer<CoolWidget>	marqueeContext::coolwdg;

marqueeContext::marqueeContext()
{
	setTitleString ( titleString );

	// Tell the context which XPM to use so the tool can properly
	// be a candidate for the 6th position on the mini-bar.
	//setImage("marqueeTool.xpm", MPxContext::kImage1 );
}

void marqueeContext::toolOnSetup ( MEvent & event)
{
	doEnterRegion(event);
	// Get the GUI's click box size
	MGlobal::executeCommand( "selectPref -query -clickBoxSize", clickBoxSize );

	view = M3dView::active3dView();
	//GLWidget* window((GLWidget*)view.widget());

	//CoolWidget *coolwdg = new CoolWidget();
	//	Create a window containing a HelixButton, if one does not already
	//	exist. Otherwise just make sure that the existing window is visible.
	//if (coolwdg.isNull()) {
		coolwdg = new CoolWidget(view.widget());
		//coolwdg->show();
	//}
	//else {
		//coolwdg->showNormal();
		//coolwdg->raise();
	//}


	/*view = M3dView::active3dView();
	GLWidget* window((GLWidget*)view.widget());
	window->setCursor(Qt::IBeamCursor);
	if(window->hasMouseTracking())
	{
		MGlobal::displayInfo("Mouse Tracking On");
	}else{
		MGlobal::displayInfo("Mouse Tracking Off");
	}
	
	QCursor mayaTip = window->cursor();
	QPoint tipPoint = mayaTip.pos();
	cout << "MOUSE MONITOR" << endl;
	cout<< tipPoint.x() << " - " << tipPoint.y() << endl;

	//window->grabMouse();
	window->resize(800,600);
    //window->show();*/

	// Get windows mouse hover
	/*view = M3dView::active3dView();
	mayaWin = view.window();
    orgProc = (WNDPROC)SetWindowLongPtr(mayaWin, GWLP_WNDPROC, (LONG)myWinProc);*/

	//SetWindowsHookEx(WH_MOUSE, mouseLoop, (HINSTANCE)mayaWin, NULL);

	MGlobal::displayInfo("Tool Setup");
	setHelpString( helpString );
}

void marqueeContext::toolOffCleanup()
{
	MGlobal::displayInfo("Tool Cleanup");
	marqueeContext::cleanup();
}

// Begin marquee drawing (using OpenGL)
// Get the start position of the marquee 
MStatus marqueeContext::doPress( MEvent & event )
{
	MSelectionList			incomingList, marqueeList;

	//setCursor( MCursor::editCursor);
	// Figure out which modifier keys were pressed, and set up the
	// listAdjustment parameter to reflect what to do with the selected points.
	if (event.isModifierShift() || event.isModifierControl() ) {
		if ( event.isModifierShift() ) {
			if ( event.isModifierControl() ) {
				MGlobal::displayInfo("Button Shift+Control Press");
				// both shift and control pressed, merge new selections
				listAdjustment = MGlobal::kAddToList;
			} else {
				MGlobal::displayInfo("Button Shift Press");
				setCursor( MCursor::crossHairCursor);
				// shift only, xor new selections with previous ones
				listAdjustment = MGlobal::kXORWithList;
			}
		} else if ( event.isModifierControl() ) {
			MGlobal::displayInfo("Button Control Press");
			// control only, remove new selections from the previous list
			listAdjustment = MGlobal::kRemoveFromList; 
		}
	} else {
		MGlobal::displayInfo("Button No Keys Press");
		listAdjustment = MGlobal::kReplaceList;
	}

	// Only select on middle mouse event
	if(event.mouseButton() == MEvent::kMiddleMouse)
	{
		// Extract the event information
		event.getPosition( pressX, pressY );

		//clickBoxSize = 100;
		// Set the selection surface area
		int halfClickBoxSize = clickBoxSize / 2;
		pressX -= halfClickBoxSize;
		pressY -= halfClickBoxSize;
		releaseX = pressX + clickBoxSize;
		releaseY = pressY + clickBoxSize;

		float size = 1;

		// Enable OpenGL drawing on viewport
		view = M3dView::active3dView();

		// to draw the selection rectangle within maya, we need to		
		// draw a custom overlay in the viewport in which the tool is being used.
		// This means we have to begin and end the rendering of an overlay
		// between which we can include all that we want to draw
		view.beginOverlayDrawing();

			// clear the overlay planes			
			//view.clearOverlayPlane();

			// Set up the orthographic projection matix			
			glMatrixMode( GL_PROJECTION );			
			glLoadIdentity();			
			gluOrtho2D( 0.0f, view.portWidth(),0.0f, view.portHeight() );			
			glMatrixMode( GL_MODELVIEW );			
			glLoadIdentity();

			view.beginGL(); 	// Start openGL

			// Push the color settings
			glPushAttrib( GL_CURRENT_BIT | GL_POINT_BIT | GL_LINE_BIT);

			// Set line width			
			/*glLineWidth( 1.0 );
			// Draw  rectangle			
			glBegin( GL_LINE_LOOP );
			glColor3f(1.0f, 1.0f, 0.0f);
			glVertex2i( pressX, pressY );				
			glVertex2i( releaseX, pressY );				
			glVertex2i( releaseX, releaseY );				
			glVertex2i( pressX, releaseY );			
			glEnd(); // end of rectangle*/

			glBegin(GL_QUADS);	
			glColor3f(0.0f, 1.0f, 0.0f);
			glVertex3f( 0.934f, 0.476f, 0.679f );			
			glVertex3f(  1.098f, 0.479f, 0.357f );
			glVertex3f( 1.231f, 0.405f,  0.400f );	
			glVertex3f(  1.047f, 0.405f,  0.761f );			
			glEnd(); // end of quad	
			
			// end of POINTS
			glPointSize( 10 ) ;	// set point size..can be 1-64
			glDisable ( GL_POINT_SMOOTH );  // Draw square "points"
			glBegin( GL_POINTS );
			glColor3f(1.0f, 0.0f, 0.0f);
			//view.setDrawColor( MColor(0.8f, 0.0f, 0.0f) );	// Set color as desired
			// Now draw the points       
			glVertex3f( 1.098f, 0.476f, -0.357f) ;  // these are x,y,z positions of point to draw
			// etc...
			glEnd();	// end of points

			glLineWidth( 5.0 );
			glBegin(GL_LINES);
			glColor3f(0.0f, 0.0f, 1.0f);
		    glVertex3d(-size, 0.0, size);
			glVertex3d(size, 0.0, size);
	        
			glVertex3d(-size, 0.0, -size);
			glVertex3d(size, 0.0, -size);
	        
			glVertex3d(size, 0.0, -size);
			glVertex3d(size, 0.0, size);

			glVertex3d(-size, 0.0, -size);
			glVertex3d(-size, 0.0, size);
			glEnd(); // end of lines

			glPopAttrib();

			view.endGL();		// End openGL

		// done drawing our overlay on the viewport		
		view.endOverlayDrawing();

		MGlobal::displayInfo("Middle Mouse Button");
		// Save the state of the current selections.  The "selectFromSceen"
		// below will alter the active list, and we have to be able to put it back.
		MGlobal::getActiveSelectionList(incomingList);

		// If we have a zero dimension box, just do a point pick
		//if ( abs(pressX - last_x) < 2 && abs(pressY - last_y) < 2 ) {
		// This will check to see if the active view is in wireframe or not.
		MGlobal::SelectionMethod selectionMethod = MGlobal::selectionMethod();

		//MGlobal::selectFromScreen( pressX, pressY, MGlobal::kReplaceList, selectionMethod );
		MGlobal::selectFromScreen( pressX, pressY, releaseX, releaseY, MGlobal::kReplaceList, selectionMethod );
		//}

		// Get the list of selected items
		MGlobal::getActiveSelectionList(marqueeList);

		// Restore the active selection list to what it was before
		// the "selectFromScreen"
		MGlobal::setActiveSelectionList(incomingList, MGlobal::kReplaceList);

		// Update the selection list as indicated by the modifier keys.
		MGlobal::selectCommand(marqueeList, listAdjustment);
	}

	return MS::kSuccess;		
}

// Drag out the marquee (using OpenGL)
MStatus marqueeContext::doDrag( MEvent & event )
{
	// Extract the event information
	event.getPosition( pressX, pressY );

	cout << "MOUSE MONITOR MAYA" << endl;
	cout << pressX << " - " << pressY <<endl; 

	// Only select on middle mouse event
	if(event.mouseButton() == MEvent::kMiddleMouse)
	{

	}

	MGlobal::displayInfo("doDrag Mouse Button");
	return MS::kSuccess;		
}

// Selects objects within the marquee box.
MStatus marqueeContext::doRelease( MEvent & event )
{
	// Only select on middle mouse event
	if(event.mouseButton() == MEvent::kMiddleMouse)
	{
		// Clear the overlay plane & restore from overlay drawing	
		view.refresh();
		//view.beginOverlayDrawing();		
		//view.clearOverlayPlane();	
		//view.endOverlayDrawing();
	}

	MGlobal::displayInfo("doRelease Mouse Button");
	return MS::kSuccess;		
}

MStatus marqueeContext::doEnterRegion( MEvent & event)
{
	MGlobal::displayInfo("doEnter Region");
	return setHelpString( helpString );
}

// returns the classname so maya will properly bind 
// the tool name to the associated mel scripts
void marqueeContext::getClassName( MString & name ) const 
{
	name.set( "marqueeContext" );
}

//////////////////////////////////////////////
// Command to create contexts
//////////////////////////////////////////////
class marqueeContextCmd : public MPxContextCommand
{
public:	
						marqueeContextCmd();
	virtual MPxContext*	makeObj();
	static	void*		creator();
};

marqueeContextCmd::marqueeContextCmd() 
{

}

MPxContext* marqueeContextCmd::makeObj()
{
	return new marqueeContext();
}

void* marqueeContextCmd::creator()
{
	return new marqueeContextCmd;
}

//////////////////////////////////////////////
// plugin initialization
//////////////////////////////////////////////
MStatus initializePlugin( MObject obj )
{
	MStatus		status;
	MFnPlugin	plugin( obj, "Miguel A Santiago Jr : DigiTeck3D", "1.0", "Any");

	status = plugin.registerContextCommand( "marqueeToolContext", marqueeContextCmd::creator );
	return status;
}

MStatus uninitializePlugin( MObject obj )
{
	MStatus		status;
	MFnPlugin	plugin( obj );

	status = plugin.deregisterContextCommand( "marqueeToolContext" );

	//	Make sure that there is no UI left hanging around.
	marqueeContext::cleanup();

	return status;
}